#
# spec file for package ctags
#
# Copyright (c) 2023 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


Name:           ctags
Version:        5.8
Release:        13.3
Summary:        A Program to Generate Tag Files for Use with vi and Other Editors
License:        GPL-2.0-or-later
Group:          Development/Tools/Navigators
URL:            http://ctags.sourceforge.net/
Source0:        http://downloads.sourceforge.net/project/%{name}/%{name}/%{version}/%{name}-%{version}.tar.gz
# No resources to make this patch work with ctags-5.8 (applied to ctags-5.7)
# Anyone is welcome to make it work again.
Source2:        ctags-ycp-parser.diff
Patch1:         0001-Mixing-with-anjuta-tags-https-git.gnome.org-browse-a.patch
Patch2:         0002-Making-inline-behave-like-an-attribute.-Fixes-1.patch
Patch3:         0003-Treat-typename-as-an-attribute.patch
Patch4:         0004-parseReturnType-should-start-from-the-first-non-brac.patch
Patch5:         0005-Ensuring-a-space-is-printed-in-return-type-AFTER-the.patch
Patch6:         0006-Prevent-C-static_assert-from-stopping-parsing.patch
Patch7:         0007-c-Handle-C-11-noexcept.patch
Patch8:         0008-c-Properly-parse-C-11-override-and-final-members.patch
Patch9:         0009-Parse-C-11-enums-with-type-specifier.patch
Patch10:        0010-Parse-C-11-classed-enums.patch
Patch11:        0011-Handle-template-expressions-that-may-use-the-or-oper.patch
Patch12:        0012-Make-sure-we-don-t-throw-things-away-while-collectin.patch
Patch13:        0013-C-mitigate-matching-error-on-generics-containing-an-.patch
Patch14:        0014-fix-wrongly-interpreted-in-template.patch
Patch15:        0015-Added-constexpr-as-keyword.patch
Patch16:        0016-CVE-2014-7204.patch
Patch17:        0017-Go-language-support.patch
Patch18:        0018-SUSE-man-page-changes.patch
Patch19:        0019-Do-not-include-build-time-in-binary.patch
Patch20:        ctags-gcc11.patch
Patch21:        CVE-2022-4515.patch
BuildRequires:  update-alternatives
Requires(pre):  update-alternatives
Requires(post): update-alternatives
Requires(post): coreutils
Provides:       arduino-ctags

%description
CTags (from Darren Hiebert) generates tag files from source code in C,
C++, Eiffel, Fortran, and Java to be used with vi and its derivatives,
Emacs, and several other editors.

%prep
%setup -q
%autopatch -p1

%build
%configure
%make_build

%install
# Makefile ignores DESTDIR ...
%make_install \
  prefix=%{buildroot}%{_prefix} \
  bindir=%{buildroot}%{_bindir} \
  mandir=%{buildroot}%{_mandir}

mv %{buildroot}%{_bindir}/ctags{,-exuberant}
mv %{buildroot}%{_mandir}/man1/ctags{,-exuberant}.1

mkdir -p %{buildroot}%{_sysconfdir}/alternatives/
ln -s %{_sysconfdir}/alternatives/ctags              %{buildroot}%{_bindir}/ctags
ln -s %{_sysconfdir}/alternatives/ctags.1%{ext_man}  %{buildroot}%{_mandir}/man1/ctags.1%{ext_man}

%post
test -L %{_bindir}/ctags || rm -f %{_bindir}/ctags
update-alternatives --install %{_bindir}/ctags ctags %{_bindir}/ctags-exuberant 20 \
  --slave %{_mandir}/man1/ctags.1.gz ctags.1 %{_mandir}/man1/ctags-exuberant.1.gz

%postun
if [ ! -f %{_bindir}/ctags-exuberant ]; then
  update-alternatives --remove ctags %{_bindir}/ctags-exuberant
fi

%files
%license COPYING
%doc EXTENDING.html FAQ README
%{_bindir}/ctags
%{_bindir}/ctags-exuberant
%{_mandir}/man1/ctags-exuberant.1%{ext_man}
%{_mandir}/man1/ctags.1%{ext_man}
%ghost %{_sysconfdir}/alternatives/ctags
%ghost %{_sysconfdir}/alternatives/ctags.1%{ext_man}

%changelog
* Thu Feb  2 2023 Dirk Müller <dmueller@suse.com>
- mark u-a generics as non-%%ghost
* Wed Jan 18 2023 Adam Majer <adam.majer@suse.de>
- CVE-2022-4515.patch: fixes arbitrary command execution via
  a tag file with a crafted filename (bsc#1206543, CVE-2022-4515)
- Stop resetting ctags update-alternative priority back to auto.
  These are admin settings.
- Remove u-a links in the correct scriptlet
* Mon Sep 27 2021 Michal Suchanek <msuchanek@suse.com>
- Fix build with gcc 11
  + ctags-gcc11.patch
* Tue May  4 2021 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Revert ghosting changes, fix real cause for build failure
  instead (symlinks pointing to itself).
- Clean spec file: remove historic cruft, use %%license for COPYING,
  use %%make_build/%%make_install macros
* Thu Jan 28 2021 Adam Majer <adam.majer@suse.de>
- Ship symlinks that are not to be ghosted in the first place
* Mon Jan 11 2021 Michal Suchanek <msuchanek@suse.com>
- Fix build on Tumbleweed - do not ship ghost symlinks.
* Sun Feb 26 2017 msuchanek@suse.com
- merge with arduino-ctags https://github.com/arduino/ctags
  * provide arduino-ctags
  * redo patch queue http://repo.or.cz/git-browser/by-commit.html?r=arduino-ctags/SUSE.git
  + new patches
    0001-Mixing-with-anjuta-tags-https-git.gnome.org-browse-a.patch
  - add vala support
  - update many existing tags with new features
    0002-Making-inline-behave-like-an-attribute.-Fixes-1.patch
    0003-Treat-typename-as-an-attribute.patch
    0004-parseReturnType-should-start-from-the-first-non-brac.patch
    0005-Ensuring-a-space-is-printed-in-return-type-AFTER-the.patch
    0006-Prevent-C-static_assert-from-stopping-parsing.patch
    0007-c-Handle-C-11-noexcept.patch
    0008-c-Properly-parse-C-11-override-and-final-members.patch
    0009-Parse-C-11-enums-with-type-specifier.patch
    0010-Parse-C-11-classed-enums.patch
    0011-Handle-template-expressions-that-may-use-the-or-oper.patch
    0012-Make-sure-we-don-t-throw-things-away-while-collectin.patch
    0013-C-mitigate-matching-error-on-generics-containing-an-.patch
    0014-fix-wrongly-interpreted-in-template.patch
    0015-Added-constexpr-as-keyword.patch
  + refreshed existing patches
    0016-CVE-2014-7204.patch
    0017-Go-language-support.patch (go-tags.patch)
    0018-SUSE-man-page-changes.patch (ctags-5.7.diff)
    0019-Do-not-include-build-time-in-binary.patch (ctags-date-time.patch)
* Thu Aug 18 2016 adam.majer@suse.de
- Own our update-alternatives symlink (bnc#994261)
- Use conflict-free symlinks instead of touch files for
  placeholders.
* Tue May 24 2016 mpluskal@suse.com
- Cleanup spec file with spec-cleaner
* Tue May 24 2016 adam.majer@suse.com
- Add missing Requires(post) on coreutils as it is using `rm`.
  (boo #976920)
* Tue Oct  7 2014 puzel@suse.com
- Fix endless loop in javascript parser
  (bnc#899486, CVE-2014-7204)
- Added CVE-2014-7204.patch
* Sat Apr 27 2013 schwab@linux-m68k.org
- Remove etags and gnuctags, provided by etags package now
- Install ctags as ctags-exuberant and provide ctags via
  update-alternatives
* Fri Mar 22 2013 mmeister@suse.com
- Added url as source.
  Please see http://en.opensuse.org/SourceUrls
* Thu Aug 16 2012 graham@andtech.eu
- go-tags.patch: Add patch supporting Go language
* Sun Sep 18 2011 jengelh@medozas.de
- Remove redundant tags/sections from specfile
* Tue Aug  2 2011 puzel@novell.com
- add etags-strcpy-overlap.patch (bnc#705308)
* Fri Jul 29 2011 puzel@novell.com
- update to ctags-5.8
  - Removed ".ml" as a Lisp extension (now OCaml) [Lisp].
  - Added support for Ant language, contributed by David Fishburn.
  - Added support for DOS Batch language, contributed by David
  Fishburn.
  - Added support for Flex (Adobe) language, contributed by David
  Fishburn.
  - Added support for MATLAB language, contributed by David Fishburn.
  - Added support for Objective Camel (OCaml), provided by Vincent
  Berthoux
  - Added support for TeX language, contributed by David Fishburn.
  - Added support for VHDL language, contributed by Nicolas Vincent
  - Added support for Pyrex/Cython declarations [Python].
  - Added support for "v" kind, for variables [Python].
  - Added support for class and member variables
  - Added support for recent enhancements to Eiffel language
  [Eiffel].
  - Added support for ASP classes, contributed by Zendhi Nagao;
  changes meaning of 'c' kind flag [ASP].
  - Added regex support when compiling with MinGW. Gnu regex module
  now included in all distributions.
  - Fixed detection of triple strings inside other strings
  - Fixed an endless loop with comments in triple strings
  - Fixed bug where functions were sometimes seen as methods
  - Fixed parsing of method parameter annotations, fix contributed by
  Paolo "blaisorblade" Giarrusso
  - Fixed parsing of global scope qualifiers in base class lists
  - Fixed bug where namespace members were given kinds corresponding
  to globals
  - Fixed parsing of "else"
  - Fixed parsing of derived enums
  - Fixed parsing of "foreach"
  - Fixed parsing of simple generic classes
  - Fixed bug with detecting identifiers inside variables
  - Fixed bug with detecting identifiers at the start of variables
  - Fixed parsing of triple single-quoted multi-line strings
- disabled YCP parser for now (does not apply)
- add ctags-date-time.patch
- use spec-cleaner
* Fri Dec 18 2009 jengelh@medozas.de
- enable parallel building
* Wed Aug 26 2009 mls@suse.de
- make patch0 usage consistent
* Fri Sep  5 2008 schwab@suse.de
- Update to etags 17.38.1.4.
* Wed Jan  9 2008 schwab@suse.de
- Update to etags 17.38.
* Tue Nov 13 2007 schwab@suse.de
- Update etags from Emacs trunk.
* Mon Nov 12 2007 bwalle@suse.de
- update to ctags 5.7
  * Added support for DIM AS [Freebasic]
  * Added support for arbitrary nesting depth [Python]
  * Added support for verbatim string literals [C#]
  * Added support for .ctags as well as ctags.cnf on Windows
  * Added support for non-extern, non-static functions returning
    wchar_t, contributed by Aaron Peromsik [C++]
  * Added support for numerous revision control systems including
    Bazaar and Mercurial
  * Added support for enums [Java]
  * Added support for multiple-level namespace declarations [C#]
  * Added .svn to list of directories ignored during recursion
    (--recurse).
  * Added support for BlitzBasic, PureBasic and FreeBasic
  * Added support for interfaces and static/public/protected/
    private functions [PHP].
  * Added support for 'package' keyword [Perl].
  * Added support for multi-line subroutine, package, and constant
    definitions [Perl].
  * Added support for optional subroutine declarations [Perl].
  * Added support for formats [Perl].
  * Added support for new convert keyword [Eiffel].
  * Added optional tags for forward variable declarations [C, C++].
  * Changed parsing of option input file (-L) to strip trailing
    white space.
  * Ignore comments mixed into definitions and declarations [Perl].
  * Fixed detecting labels with whitespace after label name [Perl]
  * Fixed parsing of generic classes/interfaces [Java]
  * Fixed misidentification of fully qualified function calls as
    labels [Perl].
  * Fixed parsing of inner classes [Python]
  * Fixed line continuation [Python]
  * Fixed parsing of annotations [Java]
  * Fixed block-comment parsing [Verilog]
  * Fixed typo in man page
  * Fixed missing chunk of text in man page and over-use of hyphens
    in UTF-8 locales
  * Fixed parsing of ` as a method name [Ruby].
  * Fixed parsing of keywords in string literals [Ruby]
  * Fixed potential segmentation violation
  * Fixed parsing of destructors with whitespace after the '~' [C++]
  * Fixed default access of unions to be public [C++]
  * Fixed various memory leaks, mostly contributed by Dmitry Antipov.
  * Fixed parsing of `define [Verilog]
  * Fixed crashes involving '/' [Verilog]
  * Fixed compilation problem on MinGW
  * Fixed generation of HTML-formatted man page
  * Fixed recognition of Python scripts having '#!/usr/bin/python'
    as first line
  * Fixed parsing of Fortran comment-to-end-of-line with no newline
    before EOF [Debian]
  * Fixed parsing of << [C/C++]
  * Fixed parsing of fully-qualified type names [Java]
  * Fixed handling of lone carriage-return characters in file
* Tue Feb  6 2007 schwab@suse.de
- Update etags from CVS.
* Tue Jan  9 2007 schwab@suse.de
- More etags fixes from CVS.
* Sat Dec 30 2006 schwab@suse.de
- Fix handling of relative names in #line.
* Wed Dec 20 2006 schwab@suse.de
- Update etags from CVS.
* Tue May 30 2006 schwab@suse.de
- Update to ctags 5.6
  * Reformatted code for independence of tab stop setting.
  * Changed default configuration to disable installation of etags links.
  * Changed --langmap to first unmap each supplied extension from other
    languages.
  * Added support for ASP constants [ASP, Patch #961842].
  * Added support for GNU make extensions [Make].
  * Added .mk as extension recognized as a make language file [Make].
  * Added missing help for list-maps options [Bug #1201826].
  * Added new extension field "typeref" [thanks to Bram Moolenaar].
  * Extended functionality of Ruby parser with patch from Elliot Hughes
    [Ruby].
  * Fixed creation of TAGS file with etags-include but no files [Bug
    [#941233]].
  * Fixed problem reading last line of list file (-L) without final
    newline.
  * Fixed infinite loop that could occur on files without final newline
    [C, Java].
  * Fixed incorrect tag for first field of table [SQL].
  * Fixed missing tags for functions beginning with underscore [Sh].
  * Fixed missing tags for functions with variable arg list [C, Bug
    [#1201689]].
  * Fixed parsing problem with parentheses in argument list [C, Bug
    [#1085585]].
  * Fixed problem in preprocessor directive handling [C, Bug #1086609].
- Update etags to 17.17.
* Tue Mar  7 2006 od@suse.de
- wrote and added a YCP parser based on the C code parser
  (ctags-5.5.4-ycp-parser.diff)
* Wed Jan 25 2006 mls@suse.de
- converted neededforbuild to BuildRequires
* Wed Dec  7 2005 od@suse.de
- Added option -A/--allow-duplicates to gnuctags
* Wed Dec  7 2005 od@suse.de
- Don't strip etags/gnuctags
* Tue Oct 18 2005 schwab@suse.de
- Update to etags 17.14.
* Mon Nov 15 2004 schwab@suse.de
- Update to etags 17.5.
* Tue May  4 2004 schwab@suse.de
- Update to ctags 5.5.4.
* Sat Jan 10 2004 adrian@suse.de
- add %%defattr
* Fri Sep 26 2003 schwab@suse.de
- Update to ctags 5.5.2.
* Tue Aug  5 2003 schwab@suse.de
- Update to ctags 5.5.1.
* Wed Apr  2 2003 schwab@suse.de
- Update to ctags 5.5.
- Update to etags 16.56.
* Tue Sep 17 2002 ro@suse.de
- removed bogus self-provides
* Tue Sep 17 2002 schwab@suse.de
- Update to ctags 5.3.1.
* Thu Jul 18 2002 schwab@suse.de
- Update to ctags 5.3.
* Fri Dec 14 2001 schwab@suse.de
- Update ctags to 5.1.
- Update etags to 14.21.
* Thu May 18 2000 schwab@suse.de
- Use BuildRoot.
- Remove etags.c changes again.
* Wed May 17 2000 kasal@suse.cz
- strip etags/gnuctags
- Fixed etags.c changes
* Mon May 15 2000 schwab@suse.de
- Discard etags.c changes (#2772).
* Mon Jan 31 2000 werner@suse.de
- Really rename gctags to gnuctags
* Mon Jan 31 2000 kasal@suse.cz
- fixed spec file to really use the *dif files
- gctags renamed to gnuctags
- rewrote the NOTEs in the manpages and added
  "see also (gnu)ctags(1), gctags(1) and global(1)"
- etags.c now uses #ifdef's
* Wed Jan 26 2000 werner@suse.de
- New ctags version 3.3.1
  * Disable etags (is usable with `ctags -e')
- Add etags/ctags from Emacs sources
  * rename this ctags with gctags
- Make some NOTEs within the manual pages of both packages
- Add a README.SuSE to explain why we have two versions
* Mon Sep 13 1999 bs@suse.de
- ran old prepare_spec on spec file to switch to new prepare_spec.
* Wed Sep  8 1999 schwab@suse.de
- specfile cleanup
* Tue Jun 15 1999 uli@suse.de
- update -> 3.2.2
* Sun Apr 13 1997 florian@suse.de
- new package with ctags 1.5 (used to be together with vim.tgz)
